#!/usr/bin/env python3
import argparse
import base64
import sys

def base64_encode_file(file_path):
    """Reads a file with UTF-8 encoding and returns its Base64 encoded string."""
    try:
        with open(file_path, 'r', encoding='utf-8') as f:
            content = f.read()
        # Encode the content to UTF-8 bytes and then to Base64 (as ASCII string)
        return base64.b64encode(content.encode('utf-8')).decode('ascii')
    except Exception as e:
        sys.exit(f"Error reading file {file_path}: {e}")

def replace_placeholders(xml_content, js_base64, css_base64):
    """Replaces the placeholders in the XML content with the provided Base64 strings."""
    xml_content = xml_content.replace(">>>contentJS<<<", js_base64)
    xml_content = xml_content.replace(">>>contentCSS<<<", css_base64)
    return xml_content

def parse_arguments():
    """Parses command-line arguments."""
    parser = argparse.ArgumentParser(
        description="Replace placeholders in an XML file with Base64-encoded JS and CSS."
    )
    parser.add_argument("input_xml", help="Path to the input XML file")
    parser.add_argument("js_file", help="Path to the JavaScript file")
    parser.add_argument("css_file", help="Path to the CSS file")
    parser.add_argument("output_xml", nargs="?", default=None,
                        help="Path for the output XML file (if not provided, input file will be overwritten)")
    return parser.parse_args()

def main():
    args = parse_arguments()
    input_xml = args.input_xml
    js_file = args.js_file
    css_file = args.css_file
    output_xml = args.output_xml if args.output_xml else input_xml

    try:
        with open(input_xml, 'r', encoding='utf-8') as f:
            xml_content = f.read()
    except Exception as e:
        sys.exit(f"Error reading XML file {input_xml}: {e}")

    js_base64 = base64_encode_file(js_file)
    css_base64 = base64_encode_file(css_file)

    updated_xml = replace_placeholders(xml_content, js_base64, css_base64)

    try:
        with open(output_xml, 'w', encoding='utf-8') as f:
            f.write(updated_xml)
        print(f"Updated XML written to: {output_xml}")
    except Exception as e:
        sys.exit(f"Error writing output file {output_xml}: {e}")

if __name__ == "__main__":
    main()
